import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

enum RewardEvent {
  onLoadAd,
  onShowAd,
  onFailedAd,
  onCompleteAd,
  onOpenAd,
  onCloseAd,
  onClickAd,
  onSkippedAd,
}

typedef void FlutterRewardListener(RewardEvent event, String zoneId);

class FlutterBaseReward{
  static const _DEFAULT_CHANNELNAME = "com.bidmad.sdk.reward/Flutter";
  String _channelNm;
  MethodChannel _platform;
  FlutterRewardListener _listener;

  Map<String, RewardEvent> rewardEventMap = <String, RewardEvent>{
    'onLoadAd': RewardEvent.onLoadAd,
    'onShowAd': RewardEvent.onShowAd,
    'onFailedAd': RewardEvent.onFailedAd,
    'onCompleteAd': RewardEvent.onCompleteAd,
    'onOpenAd': RewardEvent.onOpenAd,
    'onCloseAd': RewardEvent.onCloseAd,
    'onClickAd': RewardEvent.onClickAd,
    'onSkippedAd': RewardEvent.onSkippedAd,
  };

  String get channelNm => _channelNm;

  FlutterBaseReward({
    @required FlutterRewardListener listener,
    String channelName
  }){
    this._listener = listener;
    this._channelNm = channelName ?? _DEFAULT_CHANNELNAME;
    _initChannel(_channelNm);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
  }

  Future<void> load() async {
    await _platform.invokeMethod('load');
  }

  Future<void> show() async {
    await _platform.invokeMethod('show');
  }

  Future<void> setUserId(String userId) async {
    await _platform.invokeMethod('setUserId', userId);
  }

  Future<void> setAdInfo(String zoneId) async {
    await _platform.invokeMethod('setAdInfo', zoneId);
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    print("flutter call.method = " + call.method);

    switch (call.method) {
      case 'onLoadAd':
      case 'onShowAd':
      case 'onFailedAd':
      case 'onCompleteAd':
      case 'onOpenAd':
      case 'onCloseAd':
      case 'onClickAd':
      case 'onSkippedAd':
        final RewardEvent rewardAdEvent = rewardEventMap[call.method];
        _listener(rewardAdEvent, call.arguments.toString());
        break;
      default:
        throw ('method not defined');
    }
  }
}